@@ -278,21 +278,28 @@ gs_page_install_app (GsPage *page,
 	}
 
 	helper = g_slice_new0 (GsPageHelper);
-	if (gs_app_get_kind (app) == AS_COMPONENT_KIND_REPOSITORY)
-		helper->action = GS_PLUGIN_ACTION_INSTALL_REPO;
-	else
-		helper->action = GS_PLUGIN_ACTION_INSTALL;
 	helper->app = g_object_ref (app);
 	helper->page = g_object_ref (page);
 	helper->cancellable = g_object_ref (cancellable);
 	helper->interaction = interaction;
 	helper->propagate_error = TRUE;
 
-	plugin_job = gs_plugin_job_newv (helper->action,
-					 "interactive", (interaction == GS_SHELL_INTERACTION_FULL),
-					 "propagate-error", helper->propagate_error,
-					 "app", helper->app,
-					 NULL);
+	if (gs_app_get_kind (app) == AS_COMPONENT_KIND_REPOSITORY) {
+		helper->action = GS_PLUGIN_ACTION_INSTALL_REPO;
+		plugin_job = gs_plugin_job_manage_repository_new (helper->app,
+								  GS_PLUGIN_MANAGE_REPOSITORY_FLAGS_INSTALL |
+								  ((interaction == GS_SHELL_INTERACTION_FULL) ?
+								   GS_PLUGIN_MANAGE_REPOSITORY_FLAGS_INTERACTIVE : 0));
+		gs_plugin_job_set_propagate_error (plugin_job, helper->propagate_error);
+	} else {
+		helper->action = GS_PLUGIN_ACTION_INSTALL;
+		plugin_job = gs_plugin_job_newv (helper->action,
+						 "interactive", (interaction == GS_SHELL_INTERACTION_FULL),
+						 "propagate-error", helper->propagate_error,
+						 "app", helper->app,
+						 NULL);
+	}
+
 	gs_plugin_loader_job_process_async (priv->plugin_loader,
 					    plugin_job,
 					    helper->cancellable,
@@ -444,10 +451,17 @@ gs_page_remove_app_response_cb (GtkDialog *dialog,
 		return;
 
 	g_debug ("uninstall %s", gs_app_get_id (helper->app));
-	plugin_job = gs_plugin_job_newv (helper->action,
-					 "interactive", TRUE,
-					 "app", helper->app,
-					 NULL);
+	if (gs_app_get_kind (helper->app) == AS_COMPONENT_KIND_REPOSITORY) {
+		helper->action = GS_PLUGIN_ACTION_REMOVE_REPO;
+		plugin_job = gs_plugin_job_manage_repository_new (helper->app,
+								  GS_PLUGIN_MANAGE_REPOSITORY_FLAGS_REMOVE |
+								  GS_PLUGIN_MANAGE_REPOSITORY_FLAGS_INTERACTIVE);
+	} else {
+		plugin_job = gs_plugin_job_newv (helper->action,
+						 "interactive", TRUE,
+						 "app", helper->app,
+						 NULL);
+	}
 	gs_plugin_loader_job_process_async (priv->plugin_loader, plugin_job,
 					    helper->cancellable,
 					    gs_page_app_removed_cb,
@@ -466,17 +480,24 @@ gs_page_remove_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 	GtkWidget *remove_button;
 	GtkStyleContext *context;
 
+	/* Is the app actually removable? */
+	if (gs_app_has_quirk (app, GS_APP_QUIRK_COMPULSORY))
+		return;
+
 	/* pending install */
 	helper = g_slice_new0 (GsPageHelper);
-	if (gs_app_get_kind (app) == AS_COMPONENT_KIND_REPOSITORY)
-		helper->action = GS_PLUGIN_ACTION_REMOVE_REPO;
-	else
-		helper->action = GS_PLUGIN_ACTION_REMOVE;
+	helper->action = GS_PLUGIN_ACTION_REMOVE;
 	helper->app = g_object_ref (app);
 	helper->page = g_object_ref (page);
 	helper->cancellable = cancellable != NULL ? g_object_ref (cancellable) : NULL;
 	if (gs_app_get_state (app) == GS_APP_STATE_QUEUED_FOR_INSTALL) {
 		g_autoptr(GsPluginJob) plugin_job = NULL;
+
+		/* cancel any ongoing job, this allows to e.g. cancel pending
+		 * installations, updates, or other ops that may have been queued
+		 * in the plugin loader (due to reaching the max parallel ops allowed) */
+		g_cancellable_cancel (gs_app_get_cancellable (app));
+
 		plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
 						 "interactive", TRUE,
 						 "app", app,
